# -*- coding: utf-8 -*-
""" peltak package. """
from __future__ import absolute_import, unicode_literals
from os.path import abspath, dirname

__version__ = '0.21.1'
PKG_DIR = abspath(dirname(__file__))
