# -*- coding: utf-8 -*-
""" Simple command line form/wizard implementation. """
from __future__ import absolute_import, unicode_literals
from .core import Field, Form
__all__ = [
    'Field',
    'Form'
]
